CMGraphFrame = { } ;
function CMGraphFrame:new(o)
	o = o or {}
	setmetatable(o, self)
	self.__index = self
	return o
end

function CMGraphFrame:init(
	chartMe, const, config, measure, title, bgr, bgg, bgb, bga, barr, barg, barb, cmInterfacePanel, offset
)

	self.chartMe = chartMe
	self.const = const
	self.config = config
	self.measure = measure
	self.sizeX = self.config.frameSizeX
	self.sizeY = self.config.frameSizeY
	self.title = title
	self.bgr = bgr
	self.bgg = bgg
	self.bgb = bgb
	self.bga = bga
	self.barr = barr
	self.barg = barg
	self.barb = barb
	self.hidden = true


	local me = self

	self.barFrames = {}
	self.barTextures = {}
	--parent empty frame (may contain multiple frames in the future)
	self.mainFrame = CreateFrame("Frame", "Frame"..measure, UIParent)
	self.mainFrame:SetFrameStrata("LOW")
	self.mainFrame:SetMovable(true)
	self.mainFrame:SetResizable(true)
	self.mainFrame:EnableMouse(true)
	self.mainFrame:SetScript("OnDragStart", self.mainFrame.StartMoving)
	self.mainFrame:SetScript("OnDragStop", self.mainFrame.StopMovingOrSizing)
	self.mainFrame:SetSize(self.sizeX, self.sizeY + self.const.TITLE_BAR_HEIGHT) 
	self.mainFrame:SetPoint("TOPLEFT", 100, 0-offset)
	self.mainFrame.texture = self.mainFrame:CreateTexture() 
	self.mainFrame.texture:SetAllPoints(self.mainFrame)
	self.mainFrame.texture:SetColorTexture(bgr,bgg,bgb,bga) 
	self.mainFrame:RegisterForDrag("LeftButton");
	self.mainFrame.background = self.mainFrame.texture

	--add a title frame, texture and text

	self.titleFrame = CreateFrame("Button", "MainTitle"..measure, self.mainFrame)
	self.titleFrame:SetSize(self:getGraphWidth(), self.const.TITLE_BAR_HEIGHT)
	--self.titleFrame:SetMovable(true)
	--self.titleFrame:EnableMouse(true)
	--self.titleFrame:SetScript("OnDragStart", self.mainFrame.StartMoving)
	--self.titleFrame:SetScript("OnDragStop", self.mainFrame.StopMovingOrSizing)
	self.titleFrame:SetPoint("TOP")
	self.titleFrame:SetBackdropBorderColor(0,0,0,1)
	self.titleFrame:RegisterForDrag("LeftButton");


	self.titleFrame.texture = self.titleFrame:CreateTexture() 
	self.titleFrame.texture:SetAllPoints(self.titleFrame)
	self.titleFrame.texture:SetColorTexture(0.4,0,0,0.4)	

	self.titleFrame.text = self.titleFrame:CreateFontString(nil,"ARTWORK","GameFontNormal")
	self.titleFrame.text:SetAllPoints(self.titleFrame)
	self.titleFrame.text:SetTextColor(1,1,1,1)
	self.titleFrame.text:SetJustifyV("MIDDLE")
	self.titleFrame.text:SetJustifyH("LEFT", 5)
	self.titleFrame.text:SetText(self.title)
	


	--add a status frame, texture and text
	self.statusFrame = CreateFrame("Button", "Status", self.mainFrame)
	self.statusFrame:SetSize(self:getGraphWidth(), self.const.STATUS_BAR_HEIGHT)
	self.statusFrame:SetPoint("TOP", 0, 0-self.const.STATUS_BAR_HEIGHT)
	self.statusFrame:SetBackdropBorderColor(1,0,0,1)

	self.statusFrame.text = self.statusFrame:CreateFontString(nil,"ARTWORK","GameFontNormal")
	self.statusFrame.text:SetAllPoints(self.statusFrame)
	self.statusFrame.text:SetTextColor(1,1,1,1)
	self.statusFrame.text:SetJustifyV("MIDDLE")
	self.statusFrame.text:SetJustifyH("LEFT", 15)
	self.statusFrame.text:SetText("")

	self.statusFrame.texture = self.statusFrame:CreateTexture() 
	self.statusFrame.texture:SetAllPoints(self.statusFrame)
	self.statusFrame.texture:SetColorTexture(0,0,0,0)
	--self.statusFrame.text:SetPoint()

	--add an options button
	self.optionsFrame = CreateFrame("Button", "ResizeButton", self.titleFrame)
	self.optionsFrame:SetPoint("TOPRIGHT", -16, -3)
	self.optionsFrame:SetSize(10,10)
	self.optionsFrame:EnableMouse(true)
	--self.resizeFrame:SetResizable(true)
	self.optionsFrame.texture = self.optionsFrame:CreateTexture()
	self.optionsFrame.texture:SetTexture("Interface\\Addons\\ChartMe\\icons\\cog.tga")
	--self.optionsFrame.texture:SetColorTexture(0,0,0,0.5)
	self.optionsFrame.texture:SetAllPoints(self.optionsFrame)
	--self.resizeFrame:SetScript("OnDragStart", self.mainFrame.StartResizing)
	--self.resizeFrame:SetScript("OnDragStart", self.mainFrame.StopResizing)
	--self.resizeFrame:RegisterForDrag("LeftButton");
	self.optionsFrame:SetScript("OnClick", function (self, button, down)
		cmInterfacePanel:forceOpen()
	end)

	self.closeFrame = CreateFrame("Button", "CloseButton", self.titleFrame)
	self.closeFrame:SetPoint("TOPRIGHT", -3, -3)
	self.closeFrame:SetSize(10,10)
	self.closeFrame:EnableMouse(true)
	--self.resizeFrame:SetResizable(true)
	self.closeFrame.texture = self.closeFrame:CreateTexture()
	self.closeFrame.texture:SetTexture("Interface\\Addons\\ChartMe\\icons\\close.tga")
	--self.closeFrame.texture:SetColorTexture(0,0,0,0.5)
	self.closeFrame.texture:SetAllPoints(self.closeFrame)
	--self.resizeFrame:SetScript("OnDragStart", self.mainFrame.StartResizing)
	--self.resizeFrame:SetScript("OnDragStart", self.mainFrame.StopResizing)
	--self.resizeFrame:RegisterForDrag("LeftButton");
	self.closeFrame:SetScript("OnClick", function (self, button, down)
		me:hide()
	end)
	
	self:initBarFrames()
end

function CMGraphFrame:show()
	self.mainFrame:Show()
	self.hidden = false
end

function CMGraphFrame:hide()
	self.mainFrame:Hide()
	local dm = {}
	for i, j in pairs(self.config.displayMeasures) do
		if (j ~= self.measure) then
			table.insert(dm, j)
		end
	end
	self.config.displayMeasures = dm
	self.hidden = true
end

function CMGraphFrame:isHidden()
	return self.hidden
end

function CMGraphFrame:initBarFrames()
	self.barFrames = {}
	self.barTextures = {}
	local count = self.config.numEntries
	local x = 0;
	self.barWidth = (self:getGraphWidth() - self.config.padding*(count+1))/count
	for i=0, count-1 do
		local minX = x+self.config.padding
		minX = self:getGraphWidth() - minX - self.barWidth
		local maxX = minX+self.barWidth
		local minY = self.config.padding
		local maxY = minY+1
		local f = CreateFrame("Frame", "f"..self.measure..i, self.mainFrame)
		local t = f:CreateTexture("t"..self.measure..i, "BACKGROUND")
		t:SetColorTexture(self.barr, self.barg, self.barb, self.config.lowerAlpha)
		t:SetAllPoints()

		minX = CMUtils.round(minX)
		minY = CMUtils.round(minY)
		maxX = CMUtils.round(maxX)
		maxY = CMUtils.round(maxY)
		if self.config.debug then
			--print(minX..','..minY..' '..maxX..','..maxY)
		end
		f:SetPoint("BOTTOMLEFT", minX, minY)
		f:SetSize(maxX-minX, maxY-minY)
		self.barFrames[i] = f
		self.barTextures[i] = t
		x = x + self.barWidth + self.config.padding
	end

end

function CMGraphFrame:getMaxDmg(entries, count)
	local maxDmg = 0
	if self.config.setMaxGraphToMaxHp then
		maxDmg = UnitHealth("player")
	end
	for i=0, (count-1) do
		local e = entries[i];
		if (e > maxDmg) then
			maxDmg = e
		end
	end
	return maxDmg
end

function CMGraphFrame:plotDmgGraph(entries, measure, count)
	local maxDmg = 0
	if (self.config.setMaxGraphToAllVisible) then
		--print ("Using all vis")
		maxDmg = self.chartMe:getMaxDmgAllVisible()
	else
		maxDmg = self:getMaxDmg(entries, count)
	end

	if (maxDmg > 0) then
		self.statusFrame.text:SetText("Scale: "..maxDmg)
	else
		self.statusFrame.text:SetText("")
	end
	--print(self.recentLog:getCount())
	for i=0, (count-1) do
		local dmg = entries[i]
		local frame = self.barFrames[i]
		local maxHeight = self:getGraphHeight()-self.config.padding*2
		local barHeight = CMUtils.round(maxHeight * dmg / maxDmg)

		if barHeight == 0 then
			barHeight = 1
		end
		local alpha = (dmg/maxDmg)*(self.config.upperAlpha-self.config.lowerAlpha) + self.config.lowerAlpha
		frame:SetAlpha(alpha)
		--print(dmg .. " with maxh " .. self:round(maxHeight).. " = ".. barHeight)

		frame:SetSize(self.barWidth, barHeight)
		--print(CMUtils.dump(e))
	end
end
function CMGraphFrame:getGraphHeight()
	--print (self.sizeY)
	return self.sizeY -- self.const.STATUS_BAR_HEIGHT
end
function CMGraphFrame:getGraphWidth()
	return self.sizeX
end
