ChartMeAddon = { } ;
function ChartMeAddon:new(o)
	o = o or {}
	setmetatable(o, self)
	self.__index = self
	return o
end

function ChartMeAddon:init()
	self.const = {}
	self.const.IN_PDMG = 1
	self.const.IN_MDMG = 2
	self.const.OUT_DMG = 3
	self.const.OUT_HEAL = 4
	self.const.RAID_OUT_DMG = 5
	self.const.RAID_OUT_HEAL = 6
	self.const.TITLE_BAR_HEIGHT = 16
	self.const.STATUS_BAR_HEIGHT = 16

	self.config = {}
	self.config.debug = true
	self.config.padding = 0
	self.config.frameSizeX = 200
	self.config.frameSizeY = 80
	self.config.numEntries = 60
	self.config.updateInterval = 1
	self.config.setMaxGraphToMaxHp = false
	self.config.setMaxGraphToAllVisible = true
	self.config.displayMeasures = {
		self.const.IN_PDMG
		,self.const.IN_MDMG
		--,self.const.OUT_DMG
		,self.const.OUT_HEAL
	}
	self.config.measure = {}
	self.config.measure[self.const.IN_PDMG] = {
		title = "Incoming Physical",
		bgr = .5,
		bgg = .5,
		bgb = .5,
		bga = .2,
		barr = 1,
		barg = 1,
		barb = 1,
		recordMeasure = function (
			myGuid, timestamp, type, hideCaster, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID,
			destName, destFlags, destRaidFlags, spellId, spellName, spellSchool, amount, overkill, school, resisted, blocked, absorbed
		)
			--print ("my guid: "..myGuid)
			if destGUID == myGuid then
				if string.match(type, "_DAMAGE") then
					--swing dmg
					if type == 'SWING_DAMAGE' then
						return amount
					--or if its physical add to physical dmg
					elseif spellSchool == 1 then
						return amount
					end
				end
			end
			return 0
		end
	}
	self.config.measure[self.const.IN_MDMG] = {
		title = "Incoming Magical",
		bgr = .5,
		bgg = .5,
		bgb = .5,
		bga = .2,
		barr = 1,
		barg = 1,
		barb = 0,
		recordMeasure = function (
			myGuid, timestamp, type, hideCaster, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID,
			destName, destFlags, destRaidFlags, spellId, spellName, spellSchool, amount, overkill, school, resisted, blocked, absorbed
		)
			if destGUID == myGuid then
				if string.match(type, "_DAMAGE") then
					if type ~= "SWING_DAMAGE" and spellSchool ~= 1 then
						return amount
					end
				end
			end
			return 0;
		end

	}
	self.config.measure[self.const.OUT_HEAL] = {
		title = "Self-Heal",
		bgr = .5,
		bgg = .5,
		bgb = .5,
		bga = .2,
		barr = 0,
		barg = 1,
		barb = 0,
		--and idea in the future is for custom charts to be able to be created within the game by editing a string like this - not currently in use
		recordMeasureStr = "if sourceGUID == myGuid and string.match(type, '_HEAL') then return amount; end",
		recordMeasure = function (
			myGuid, timestamp, type, hideCaster, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID,
			destName, destFlags, destRaidFlags, spellId, spellName, spellSchool, amount, overkill, school, resisted, blocked, absorbed
		)
			if sourceGUID == self.myGuid then
				if string.match(type, "_HEAL") then
					return amount;
				end
			end
			return 0
		end
	}
	self.config.measure[self.const.OUT_DMG] = {
		title = "Damage",
		bgr = .5,
		bgg = .5,
		bgb = .5,
		bga = .2,
		barr = 1,
		barg = 0,
		barb = 0,
		recordMeasure = function (
			myGuid, timestamp, type, hideCaster, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID,
			destName, destFlags, destRaidFlags, spellId, spellName, spellSchool, amount, overkill, school, resisted, blocked, absorbed
		)
			if sourceGUID == myGuid then
				if string.match(type, "_DAMAGE") then
					return amount
				end
			end
			return 0
		end
	}
	self.config.measure[self.const.RAID_OUT_HEAL] = {
		title = "Raid Heal",
		bgr = .5,
		bgg = .5,
		bgb = .5,
		bga = .2,
		barr = 1,
		barg = 0,
		barb = 1,
		recordMeasure = function (
			myGuid, timestamp, type, hideCaster, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID,
			destName, destFlags, destRaidFlags, spellId, spellName, spellSchool, amount, overkill, school, resisted, blocked, absorbed
		)
			if bit.band(sourceFlags, COMBATLOG_OBJECT_AFFILIATION_RAID) then
				if string.match(type, "_HEAL") then
					return amount
				end
			end
			return 0
		end
	}
	self.config.measure[self.const.RAID_OUT_DMG] = {
		title = "Raid Damage",
		bgr = .5,
		bgg = .5,
		bgb = .5,
		bga = .2,
		barr = 0,
		barg = 1,
		barb = 1,
		recordMeasure = function (
			myGuid, timestamp, type, hideCaster, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID,
			destName, destFlags, destRaidFlags, spellId, spellName, spellSchool, amount, overkill, school, resisted, blocked, absorbed
		)
			if
				bit.band(sourceFlags, COMBATLOG_OBJECT_AFFILIATION_RAID) or
				bit.band(sourceFlags, COMBATLOG_OBJECT_AFFILIATION_PARTY) or
				sourceGUID == myGuid
			then
				if string.match(type, "_DAMAGE") then
					return amount
				end
			end
			return 0
		end
	}

	self.config.lowerAlpha = 0.6
	self.config.upperAlpha = 1

	self.recentLog = RecentLog:new()
	self.recentLog:init(self.config.numEntries)
	self.totalElapsed = 0
	self.myGuid = UnitGUID("player")
	self.mainFrame = CreateFrame("Frame", "Frame", UIParent)
	self.mainFrame:SetAllPoints()
	self.mainFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	
	self.cmInterfacePanel = CMInterfacePanel:new()
	self.cmInterfacePanel:init(self, self.config)

	self.graphFrames = { }


	self:redrawGraphFrames()
	
	self:resetMeasures()



	local me = self

	--on event is for the combat log listener	
	self.mainFrame:SetScript("OnEvent", function(self, event, ...)
		local timestamp, type, hideCaster, sourceGUID, sourceName, sourceFlags, sourceFlags2, destGUID,
			destName, destFlags, destFlags2, spellIdOrSwingDmgAmount, spellName, spellSchool, amount = select(1, ...)
		me:recordEvent(
			timestamp, type, hideCaster, sourceGUID, sourceName, sourceFlags, sourceFlags2, destGUID, 
			destName, destFlags, destFlags2, spellIdOrSwingDmgAmount, spellName, spellSchool, amount
		);
	end)

	-- timer event - to refresh every 0.5 secs
	self.mainFrame:SetScript("OnUpdate", function(self, elapsed)
		me:update(elapsed)
	end)



end

function ChartMeAddon:redrawGraphFrames()
	local offset = 200
	for measure, _ in pairs(self.config.measure) do

		--print ('init frames for measure '..measure)
		if self.graphFrames[measure] == nil then
			self.graphFrames[measure] = CMGraphFrame:new()
			local conf = self.config.measure[measure]
			self.graphFrames[measure]:init(
				self, self.const, self.config, measure, conf.title, conf.bgr, conf.bgg, conf.bgb, conf.bga,
				conf.barr, conf.barg, conf.barb, self.cmInterfacePanel, offset
			)
		end
		if CMUtils.inTable(measure, self.config.displayMeasures) then
			self.graphFrames[measure]:show()
			offset = offset + self.config.frameSizeY+self.const.TITLE_BAR_HEIGHT
		else
			self.graphFrames[measure]:hide()
		end
		
	end
end

-- static --

-- static --


function ChartMeAddon:addMeasure(type, amount)
	--print ("adding ".. amount .. " to measure ".. type)
	self.eventData[type] = self.eventData[type] + amount
end
function ChartMeAddon:getMeasure(type)
	return self.eventData[type]
end
function ChartMeAddon:resetMeasures()
	self.eventData = {};
	for measure, _ in pairs(self.config.measure) do
		self.eventData[measure] = 0;
	end
end

function ChartMeAddon:getEventDataByType(type)
	local typedEntries = {}
	if self.recentLog then

		local entries = self.recentLog:getAllEvents()
		
		local count = self.recentLog:getCount()

		for i = 0, (count-1) do
			typedEntries[i] = entries[i][type]
		end
	end
	return typedEntries
end

function ChartMeAddon:refreshData()

	if self.recentLog then
		
		local count = self.recentLog:getCount()

		--print(CMUtils.dump(self.graphFrames[self.const.OUT_HEAL]))
		
		for measure, _ in pairs(self.config.measure) do
			local graphFrame = self.graphFrames[measure]
			local entries = self:getEventDataByType(measure)
			--print(CMUtils.dump(entries))
			graphFrame:plotDmgGraph(entries, measure, count)
		end

		--print ('count '..count)
	end
end

function ChartMeAddon:getMaxDmgAllVisible()
	local maxDmg = 0
	if self.recentLog then
		
		local count = self.recentLog:getCount()

		for measure, _ in pairs(self.config.measure) do
			local graphFrame = self.graphFrames[measure]
			if not graphFrame:isHidden() then
				local entries = self:getEventDataByType(measure)
				local d = graphFrame:getMaxDmg(entries, count)

				if (d > maxDmg) then
					maxDmg = d
				end
			end
		end
	end
	return maxDmg
end

function ChartMeAddon:recordEvent(
	timestamp, type, hideCaster, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID,
	destName, destFlags, destRaidFlags, spellId, spellName, spellSchool, amount,
	overkill, school, resisted, blocked, absorbed
)
	--print "event recording"
	--the people at blizz decided to not include the spellId, spellName and spellSchool if the dmg is a melee swing
	if type == 'SWING_DAMAGE' then
		absorbed = school
		blocked = overkill
		resisted = amount
		school = spellSchool
		overkill = spellName
		amount = spellId
	end
	for measure, measureConfig in pairs(self.config.measure) do
		local valToStore = 0;
		
		valToStore = measureConfig.recordMeasure(
			self.myGuid, timestamp, type, hideCaster, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID,
			destName, destFlags, destRaidFlags, spellId, spellName, spellSchool, amount,
			overkill, school, resisted, blocked, absorbed
		)
		if valToStore > 0 then
			if (self.config.debug) then
				--print("adding "..valToStore.." to measure "..measure)
			end
			self:addMeasure(measure, valToStore)
		end
	end
end

--intialises the added after 5 seconds to fix a first-time-loading problem where the charts wouldnt load without this delay
--@TODO find a cleaner way to do this
function ChartMeAddon:pauseInit(delay)
	self.pauseInitTotalElapsed = 0
	if(type(delay)~="number") then
		return false;
	end
	local waitFrame = CreateFrame("Frame","WaitFrame", UIParent)
	local me = self
	waitFrame:SetScript("onUpdate",function (self,elapse)
		me.pauseInitTotalElapsed = me.pauseInitTotalElapsed + elapse
		if (me.pauseInitTotalElapsed > 5) then
			if (me.config == nil) then
				me:init()
			end
		end
	end)
	return true
end

function ChartMeAddon:update(elapsed)
	self.totalElapsed = self.totalElapsed + elapsed
  	if self.totalElapsed > self.config.updateInterval then
    	self.totalElapsed = 0;
    	self.recentLog:add(self.eventData);
    	
    	self:refreshData()
	--ChartMeAddon["recentLog"]:display()
    	self:resetMeasures()
  end
end

chartMeAddon = ChartMeAddon:new()
chartMeAddon:pauseInit(5)


